import { NodeOrString } from "../interfaces";
import { CoordBox } from "./html";
import { NodeList as DojoNodeList, NodeListConstructor as DojoNodeListConstructor } from "../query";

declare module "../query" {
    interface NodeList<T extends Node> {
        /**
         * Attach event handlers to every item of the NodeList. Uses dojo.connect()
         * so event properties are normalized.
         */
        connect(method: string, objOrFunc: EventListener | string): this;
        connect(method: string, objOrFunc: Object, funcName: string): this;

        /**
         * Deprecated: Use position() for border-box x/y/w/h
         * or marginBox() for margin-box w/h/l/t.
         * Returns the box objects of all elements in a node list as
         * an Array (*not* a NodeList). Acts like `domGeom.coords`, though assumes
         * the node passed is each node in this list.
         */
        coords(node: NodeOrString, includeScroll?: boolean): CoordBox[];

        onblur(method: string, objOrFunc: EventListener | string): this;
        onblur(method: string, objOrFunc: Object, funcName: string): this;

        onfocus(method: string, objOrFunc: EventListener | string): this;
        onfocus(method: string, objOrFunc: Object, funcName: string): this;

        onchange(method: string, objOrFunc: EventListener | string): this;
        onchange(method: string, objOrFunc: Object, funcName: string): this;

        onclick(method: string, objOrFunc: EventListener | string): this;
        onclick(method: string, objOrFunc: Object, funcName: string): this;

        onerror(method: string, objOrFunc: EventListener | string): this;
        onerror(method: string, objOrFunc: Object, funcName: string): this;

        onkeydown(method: string, objOrFunc: EventListener | string): this;
        onkeydown(method: string, objOrFunc: Object, funcName: string): this;

        onkeypress(method: string, objOrFunc: EventListener | string): this;
        onkeypress(method: string, objOrFunc: Object, funcName: string): this;

        onkeyup(method: string, objOrFunc: EventListener | string): this;
        onkeyup(method: string, objOrFunc: Object, funcName: string): this;

        onload(method: string, objOrFunc: EventListener | string): this;
        onload(method: string, objOrFunc: Object, funcName: string): this;

        ondown(method: string, objOrFunc: EventListener | string): this;
        ondown(method: string, objOrFunc: Object, funcName: string): this;

        onmouseenter(method: string, objOrFunc: EventListener | string): this;
        onmouseenter(method: string, objOrFunc: Object, funcName: string): this;

        onmouseleave(method: string, objOrFunc: EventListener | string): this;
        onmouseleave(method: string, objOrFunc: Object, funcName: string): this;

        onmousemove(method: string, objOrFunc: EventListener | string): this;
        onmousemove(method: string, objOrFunc: Object, funcName: string): this;

        onmouseout(method: string, objOrFunc: EventListener | string): this;
        onmouseout(method: string, objOrFunc: Object, funcName: string): this;

        onmouseover(method: string, objOrFunc: EventListener | string): this;
        onmouseover(method: string, objOrFunc: Object, funcName: string): this;

        onmouseup(method: string, objOrFunc: EventListener | string): this;
        onmouseup(method: string, objOrFunc: Object, funcName: string): this;

        onsubmit(method: string, objOrFunc: EventListener | string): this;
        onsubmit(method: string, objOrFunc: Object, funcName: string): this;
    }

    interface NodeListConstructor {
        // "blur", "focus", "change", "click", "error", "keydown", "keypress",
		// "keyup", "load", "mousedown", "mouseenter", "mouseleave", "mousemove",
		// "mouseout", "mouseover", "mouseup", "submit"
        events: string[];
    }
}

declare module "./kernel" {
    interface Dojo {
        NodeList: DojoNodeListConstructor
    }
}

declare const DojoNodeList: DojoNodeListConstructor;
export = DojoNodeList;
